from fabric.tasks import Task
from dploi_fabric.db.base import DumpDatabaseTask, DownloadDatabase

class MysqlDumpDatabaseTask(DumpDatabaseTask, Task):
    """
    Dump the database (MySQL)
    """
    name = 'dump'
    database_type = 'mysql'

    def run(self, reason='unknown', compress=False, nolock=False):
        self.nolock = nolock
        return super(MysqlDumpDatabaseTask, self).run(reason, compress)

    def get_command(self, env, file_name):
        nolock = ' --lock-tables=false' if self.nolock else ''
        host = ' --host %(db_host)s' % env if hasattr(env, 'db_host') else ''
        return ('mysqldump' + nolock + host + ' --user="%(db_username)s" --password="%(db_password)s" "%(db_name)s" > ' % env) + file_name

dump = MysqlDumpDatabaseTask()
download = DownloadDatabase(dump_task=dump)



from fabric.operations import run as do_run
from fabric.api import task
from .utils import config

@task
def update():
    """
    updates a virtualenv (pip install requirements.txt)
    """
    do_run('cd %(path)s; bin/pip install -r requirements.txt' % config.sites["main"].deployment)

@task
def create():
    """
    creates a virtualenv and calls update
    """
    do_run('cd %(path)s; virtualenv . ' % config.sites["main"].deployment)
    update()



from fabric.api import env, run
from fabric.tasks import Task
from .utils import config

class SouthMigrateTask(Task):

    name = 'migrate'

    def run(self):
        config.django_manage("syncdb")
        config.django_manage("migrate")

migrate = SouthMigrateTask()


# -*- coding: utf-8 -*-
import ConfigParser

class EnvConfigParser(ConfigParser.SafeConfigParser):
    """ A config parser that can handle "namespaced" sections. Example:

    [base]
    name = base

    [base:some-env]
    name = some-env

    """

    def _concat(self, parent, child):
        return '%s:%s' % (parent, child)

    def items(self, section, raw=False, vars=None, env=None):
        items = {}
        try:
            items.update(dict(ConfigParser.SafeConfigParser.items(self, section, raw, vars)))
        except ConfigParser.NoSectionError:
            pass
        if env:
            try:
                env_items = dict(ConfigParser.SafeConfigParser.items(self, self._concat(section, env), raw, vars))
                items.update(env_items)
            except ConfigParser.NoSectionError:
                pass
        if not items:
            raise ConfigParser.NoSectionError(self._concat(section, env) if env else section)
        return tuple(items.iteritems())

    def get(self, section, option, raw=False, vars=None, env=None):
        if env and self.has_section(self._concat(section, env)):
            try:
                return ConfigParser.SafeConfigParser.get(self, self._concat(section, env), option, raw, vars)
            except ConfigParser.NoOptionError:
                if not self.has_section(section):
                    raise
        return ConfigParser.SafeConfigParser.get(self, section, option, raw, vars)

    def _get(self, section, conv, option, env=None):
        return conv(self.get(section, option, env=env))

    def getint(self, section, option, env=None):
        return self._get(section, int, option, env)

    def getfloat(self, section, option, env=None):
        return self._get(section, float, option, env)

    def getboolean(self, section, option, env=None):
        v = self.get(section, option, env=env)
        if v.lower() not in self._boolean_states:
            raise ValueError, 'Not a boolean: %s' % v
        return self._boolean_states[v.lower()]

    def has_section(self, section, env=None, strict=False):
        if not env:
            return ConfigParser.SafeConfigParser.has_section(self,section)
        return (
            (not strict and ConfigParser.SafeConfigParser.has_section(self, section)) or
            ConfigParser.SafeConfigParser.has_section(self, self._concat(section, env))
            )

    def section_namespaces(self, section):
        namespaces = []
        for s in self.sections():
            s = s.split(":")
            if s[0] == section:
                if len(s) == 1:
                    namespaces.append("main")
                else:
                    namespaces.append(s[1])
        return namespaces

    def _interpolate(self, section, option, rawval, vars):
        return rawval



import StringIO
from fabric.decorators import task
from fabric.api import run, put

from dploi_fabric.toolbox.template import render_template
from dploi_fabric.utils import config

@task
def update_config_file():
    template = 'templates/supervisor/supervisor.conf'
    output = ''
    for site, site_config in config.sites.items():
        for process_name, process_dict in site_config.processes.items():
            context_dict = site_config
            context_dict.update({'process_name': process_name, 'process_cmd': process_dict["command"], 'socket': process_dict["socket"]})
            output += render_template(template, context_dict)
    put(StringIO.StringIO(output), '%(path)s/../config/supervisor.conf' % config.sites["main"].deployment)
    update()

@task
def stop():
    for site, site_config in config.sites.items():
        for process_name, process_cmd in site_config.processes.items():
            run('sudo supervisorctl stop %s' % process_name)


@task
def start():
    for site, site_config in config.sites.items():
        for process_name, process_cmd in site_config.processes.items():
            run('sudo supervisorctl start %s' % process_name)

@task
def restart():
    for site, site_config in config.sites.items():
        for process_name, process_cmd in site_config.processes.items():
            run('sudo supervisorctl restart %s' % process_name)

@task
def status():
    """
    print status of the supervisor process
    """
    for site, site_config in config.sites.items():
        for process_name, process_cmd in site_config.processes.items():
            run('sudo supervisorctl status %s' % process_name)

@task
def add():
    for site, site_config in config.sites.items():
        for process_name, process_cmd in site_config.processes.items():
            run('sudo supervisorctl add %s' % process_name)

@task
def update():
    for site, site_config in config.sites.items():
        for process_name, process_cmd in site_config.processes.items():
            run('sudo supervisorctl update %s' % process_name)


package_name = 'dploi_fabric'
name = 'dploi-fabric'
author = 'Benjamin Wohlwend, Kristian Oellegaard, Stefan Foulis'
author_email = 'developers@divio.ch'
description = "A collection of fabric tasks"
version = __import__(package_name).__version__
project_url = 'http://github.com/dploi/%s' % name
license = 'TBA'
